@@ -25,28 +25,32 @@
 """
 import sys
 import argparse
-
 import fs
 from PyQt5 import QtWidgets, QtCore
+from PyQt5.QtGui import QIcon
 from fslib import ui_filepicker, __version__
-from fslib.utils import match_extension
-
+from fslib.utils import match_extension, WidgetImageText
+from fslib.icons import icons
 
 class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
     def __init__(self, parent=None, fs_url=u"~/", file_pattern=u'*.*', title=u'FS File Picker',
                  default_filename=None, show_save_action=False):
         super(FilePicker, self).__init__(parent)
         self.setupUi(self)
+        self.file_icon = icons('text-x-generic.png')
+        self.dir_icon = icons('folder.png')
         self.selected_dir = None
         self.filename = None
         self.setWindowTitle(title)
         self.fs_url = unicode(fs_url)
+        self.parent_url = unicode(fs_url)
         self.fs = None
         self.file_list_items = []
         self.last_index = 0
         self.default_filename = default_filename
         self.file_pattern = file_pattern
         self.show_save_action = show_save_action
+        self.button_icons()
         self.show_action()
         self.ui_FileType.setText(self.file_pattern)
         self.browse_folder()
@@ -56,6 +60,13 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
         self.action_buttons()
         self.ui_FileList.itemClicked.connect(self.show_name)
         self.ui_mkdir.clicked.connect(self.make_dir)
+        self.ui_parentdir.clicked.connect(self.parent_dir)
+
+    def button_icons(self):
+        self.ui_parentdir.setText("")
+        self.ui_parentdir.setIcon(QIcon(icons('go-top.png')))
+        self.ui_mkdir.setText("")
+        self.ui_mkdir.setIcon(QIcon(icons('folder-new.png')))
 
     def action_buttons(self):
         try:
@@ -63,14 +74,19 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
         except AttributeError:
             pass
 
-    def browse_folder(self):
+    def browse_folder(self, subdir=None):
         """
         walks through all folders and selects first directory
         """
         if self.show_save_action:
             self.ui_Action.setEnabled(True)
         self.ui_DirList.clear()
-        self.fs = fs.open_fs(self.fs_url)
+        if subdir:
+            self.parent_url = fs.path.combine(self.parent_url, subdir)
+            self.fs = fs.open_fs(self.parent_url)
+        else:
+            self.fs = fs.open_fs(self.fs_url)
+            self.parent_url = self.fs_url
         self.ui_DirList.addItem(u'.')
         for dir_path in sorted(self.fs.listdir(u'.')):
             if self.fs.isdir(dir_path):
@@ -89,24 +105,50 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
         self.ui_FileList.clearContents()
         file_type = self.ui_FileType.text()
         self.file_list_items = []
+        self.dir_list_items = []
         self.ui_FileList.setColumnCount(1)
-        self.ui_FileList.setRowCount(len(self.file_list_items))
+        self.ui_FileList.setColumnWidth(0, 400)
         self.ui_FileList.verticalHeader().setVisible(False)
         self.ui_FileList.horizontalHeader().setVisible(False)
         self.ui_FileList.setShowGrid(False)
+        self.ui_FileList.setSizeAdjustPolicy(
+            QtWidgets.QAbstractScrollArea.AdjustToContents)
 
         for item in sorted(self.fs.listdir(self.selected_dir)):
             if not self.fs.isdir(item) and match_extension(item, [file_type]):
                 self.file_list_items.append(item)
-        self.ui_FileList.setRowCount(len(self.file_list_items))
+            elif self.fs.isdir(item):
+                self.dir_list_items.append(item)
+
+        self.ui_FileList.setRowCount(len(self.file_list_items) + len(self.dir_list_items))
         index = 0
+
+        for item in self.dir_list_items:
+            self.ui_FileList.setCellWidget(index, 0, WidgetImageText(item, self.dir_icon))
+            index = index + 1
         for item in self.file_list_items:
-            self.ui_FileList.setItem(index, 0,  QtWidgets.QTableWidgetItem(item))
+            self.ui_FileList.setCellWidget(index, 0, WidgetImageText(item, self.file_icon))
             index = index + 1
         if self.last_index == 0 and not self.show_save_action:
             self.ui_FileList.clearSelection()
             if self.ui_FileList.currentItem() is not None:
                 self.ui_SelectedName.setText(self.ui_FileList.currentItem().text())
+        self.ui_FileList.resizeRowsToContents()
+        self.ui_FileList.cellClicked.connect(self.onCellClicked)
+
+    @QtCore.pyqtSlot(int, int)
+    def onCellClicked(self, row, column):
+        wparm = self.ui_FileList.cellWidget(row, column)
+        if wparm is not None:
+            if "text" in wparm.img:
+                self.ui_SelectedName.setText(wparm.text)
+
+            if "folder" in wparm.img:
+                index = self.dir_list_items.index(wparm.text) + 1
+                if index != 0:
+                    self.ui_DirList.setCurrentIndex(index)
+                    self.selection_directory(index)
+                    self.browse_folder(subdir=wparm.text)
 
     def selection_file_type(self):
         """
@@ -122,9 +164,9 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
         if ((self.filename not in (u"",  u".") and self.fs.exists(fs.path.join(self.selected_dir, self.filename)) and
                         self.filename in self.file_list_items)):
             self.ui_Action.setEnabled(True)
-            item = self.ui_FileList.findItems(self.filename, QtCore.Qt.MatchExactly)
+            all_items = self.dir_list_items + self.file_list_items
             try:
-                self.ui_FileList.selectRow(self.file_list_items.index(self.filename))
+                self.ui_FileList.selectRow(all_items.index(self.filename))
             except TypeError:
                 pass
         else:
@@ -134,18 +176,20 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
 
     def show_name(self):
         try:
-            self.filename = self.ui_FileList.currentItem().text()
+            self.filename = self.ui_SelectedName.text()
         except AttributeError:
             self.filename = None
+        if self.filename == u"":
+            self.filename = None
         self.ui_SelectedName.setText(self.filename)
         if self.fs.exists(fs.path.join(self.selected_dir, self.filename)):
             self.ui_Action.setEnabled(True)
 
-
     def cancel(self):
         """
         Action on cancel button
         """
+        self.filename = None
         self.close()
 
     def show_action(self):
@@ -173,6 +217,9 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
             else:
                 ok = QtWidgets.QMessageBox.warning(self, "New Folder", "Can't create this Folder: {}".format(new_dir))
 
+    def parent_dir(self):
+        self.browse_folder()
+
     def action(self):
         """
         Action on open / save button
@@ -192,7 +239,9 @@ class FilePicker(QtWidgets.QDialog, ui_filepicker.Ui_Dialog):
                 self.close()
         else:
             try:
-                self.filename = self.ui_FileList.currentItem().text()
+                self.filename = self.ui_SelectedName.text()
+                if self.filename == u"":
+                    self.filename = None
             except AttributeError:
                 pass
             else:
@@ -216,7 +265,7 @@ def fs_filepicker(parent=None, fs_url=u'~/', file_pattern=u'*.*', title=u'FS Fil
     fp.exec_()
     filename = None
     if fp.filename is not None:
-        filename = fs.path.combine(fp.fs_url, fs.path.join(fp.selected_dir, fp.filename))
+        filename = fs.path.combine(fp.parent_url, fp.filename)
     return filename
 
 
@@ -237,6 +286,7 @@ def main():
         print("Version:", __version__)
         sys.exit()
     app = QtWidgets.QApplication([])
+
     return fs_filepicker(parent=None, fs_url=args.fs_url, file_pattern=args.file_pattern,
                          title=args.title, default_filename=args.default_name,
                          show_save_action=args.save)
